/*
 * utils.h
 * Copyright (C) 2005-2007 Robin KAY. All rights reserved.
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 */

#ifndef UTILS_H
#define UTILS_H

#define REWRITE_FLAGS_NONE           0x00000000
#define REWRITE_FLAGS_IGN_LAST_SYM   0x00000001
#define REWRITE_FLAGS_IGN_LAST_NOENT 0x00000002
#define REWRITE_FLAGS_VPATH_ONLY     0x00000004

#define PFLAGS_COMBINE(b, t) (((b) << 16) | (t))
#define PFLAGS_EXTRACT_BASE(f) (0xffff & ((f) >> 16))
#define PFLAGS_EXTRACT_TARGET(f) (0xffff & (f))
#define PFLAGS_OVERLAY(f) (0xffff & (((f) >> 16) | (f)))

typedef struct {
  size_t len;
  char buf[PATH_MAX];
} path_buf_t;

typedef RDLNK_RETTYPE (*readlink_fptr_t)(const char *, char *, size_t);
typedef int (*path_rewrite_cb_t)(const char *, const char *, int, void *);

extern path_buf_t *path_copy(path_buf_t *, path_buf_t *);
extern path_buf_t *path_set(path_buf_t *, const char *);
extern path_buf_t *path_change(path_buf_t *, const char *);
extern char *path_get_cstr(path_buf_t *);
extern void path_rewrite_init(readlink_fptr_t);
extern int path_rewrite(path_buf_t *, int, path_rewrite_cb_t, void *);

#define ALIGN(n, a) (((n) + (a) - 1) & ~((a) - 1))
#define RECLEN(T) ALIGN(sizeof(T), _MAX_ALIGNMENT)
#define TO_HDR(T, D) (T *)(((char *)(D)) - RECLEN(T))
#define HDR_TO(T, H) (void *)(((char *)(H)) + RECLEN(T))
#define MAX(a, b) ((a) > (b) ? (a) : (b))

extern void alloc_init(size_t, size_t);
extern void *block_alloc();
extern void block_free(void *);
extern void *stack_alloc(void *, size_t);
extern void stack_free(void *);

struct tree_node_s {
  struct tree_node_s *left;
  struct tree_node_s *right;
  struct tree_node_s *next;
};
typedef struct tree_node_s tree_node_t;

typedef int (*tree_cmp_cb_t)(void *, void *);

typedef enum {
  INSERT,
  LOOKUP,
  REMOVE,
  TEST_INSERT
} tree_action_t;

extern void *tree_op(tree_node_t **, tree_cmp_cb_t, void *, tree_action_t);
extern tree_node_t *tree_linearize(tree_node_t *);

#endif
